  /**
    * @file pwm_driver_avr.c
    * @brief This module contains the implementation of the PWM driver hardware
    * function

    * @author Stefan Popovic
    *
    * @date 7/12/2016
    *
    * @version 0.1a
    *
    * @copyright GNU Public License
    *
    * Test configuration:
    *
    * MCU: ATMEGA32
    * Dev.Board: EasyAVR v7 Connectivity
    * External Clock 08.0000 MHz
    * Ext. Modules: Brushless Click Board
    * SW: MikroC PRO for AVR v6.1.2

    * NOTES:
      - Place PWM driver click board on mikroBUS slot 1
      - Turn on Lcd backlight on SW3.1
      - Turn ON the PORTC LEDs at SW3.3
      - To simulate analog input on ADC channel 6, use on-board potentiometer P3
        by connecting jumper (J4) (board specific) to PA6 (ADC channel 6 input)
      - PWM: RC1 _TIMER1_CH_B, 8 - bit Timer register
      - ADC: PA6 ADC Input Channel 6, 10 - bit
      - Output LED: PD4

    /**************************************************************************
    * Module Preprocessor Constants
    **************************************************************************/

    /**************************************************************************
    * Module Preprocessor Macros
    **************************************************************************/


    /**************************************************************************
    * Module Typedefs
    **************************************************************************/

     /**************************************************************************
     * Includes
     **************************************************************************/
     #include "built_in.h"
     #include <stdint.h>
    /**************************************************************************
    * Module Variable Definitions
    **************************************************************************/
    // LCD module connections
    sbit LCD_RS at PORTA2_bit;
    sbit LCD_EN at PORTD6_bit;
    sbit LCD_D4 at PORTC4_bit;
    sbit LCD_D5 at PORTC5_bit;
    sbit LCD_D6 at PORTC6_bit;
    sbit LCD_D7 at PORTC7_bit;

    sbit LCD_RS_Direction at DDA2_bit;
    sbit LCD_EN_Direction at DDD6_bit;
    sbit LCD_D4_Direction at DDC4_bit;
    sbit LCD_D5_Direction at DDC5_bit;
    sbit LCD_D6_Direction at DDC6_bit;
    sbit LCD_D7_Direction at DDC7_bit;
    // End LCD module connections

    uint16_t current_duty;
    uint16_t adc_rd;
    char txt1[] = "PWM driver click";
    char txt2[] = "PWM Duty";
    char txt3[] = "%";
    char txt[5];

    void MCU_Init()
    {
      ADC_Init(); // ADC Init
      PWM16bit_Init(_PWM16_PHASE_CORRECT_MODE_8BIT, _PWM16_PRESCALER_16bit_8, _PWM16_NON_INVERTED, 255, 1);  // PWM Init
      PWM16bit_Start(_TIMER1);
      PWM16bit_Change_Duty(current_duty, _TIMER1_CH_B);

      Delay_ms(100);

      current_duty = 0; // Initial value for current duty
    }
    
     void Lcd_Display()
    {
      Lcd_Cmd(_LCD_CLEAR);                // Clear display
      Lcd_Cmd(_LCD_CURSOR_OFF);           // Cursor off
      Lcd_Out(1, 1, txt1);                // Write text in first row
      Lcd_Out(2, 1, txt2);                // Write text in second row
      Lcd_Out(2, 16, txt3);               // Write text in second row
    }

    void main() 
    {
        MCU_Init();
        Lcd_Init();                        // Initialize Lcd
        Lcd_Display();                     // Display Lcd
        DDA6_bit = 0;                      // Set PORTA pin 6 as input pin for ADC
        DDD4_bit = 1;                      // Set PORTD pin 4 as output pin for PWM

        while( 1 )  // Playing with Potentiometer P3 you can control current PWM duty cycle. For visual result, look at LED PD4
        {
          uint8_t cur_duty_byte;

          // Read 10 - bit ADC value and set newly acquired 8 - bit PWM duty
          adc_rd = ADC_Read(6) & 0x0000FFFF;
          current_duty = adc_rd / 4 ;
          cur_duty_byte = LoWord(current_duty);
          PWM16bit_Change_Duty(cur_duty_byte, _TIMER1_CH_B);     // Set newly acquired duty
          cur_duty_byte = cur_duty_byte * 0.39; // 100 / 256
          ByteToStr(cur_duty_byte, txt);
          Lcd_Out(2, 11, txt);
          Delay_ms( 20 );
        }
     }

